@@ -29,18 +29,21 @@
   return(etaFix)
 }
 
-.r_resid_var <- function(mu,phiW,sizes,COMP_nu,NB_shape,famfam) {switch(famfam,
-                                                          gaussian = rnorm(length(mu),mean=mu,sd=sqrt(phiW)),
-                                                          poisson = rpois(length(mu),mu),
-                                                          binomial = rbinom(length(mu),size=sizes,prob=mu),
-                                                          Gamma = rgamma(length(mu),shape= mu^2 / phiW, scale=phiW/mu), ## ie shape increase with prior weights, consistent with Gamma()$simulate / spaMM_Gamma()$simulate
-                                                          COMPoisson = sapply(mu, function(muv) {
-                                                            lambda <- family$linkfun(muv,log=FALSE)
-                                                            .COMP_simulate(lambda=lambda,nu=COMP_nu)
-                                                          }),
-                                                          negbin = MASS::rnegbin(mu, theta=NB_shape),
-                                                          stop("(!) random sample from given family not yet implemented")
-)} ## vector-valued function from vector input
+.r_resid_var <- function(mu,phiW,sizes,COMP_nu,NB_shape,zero_truncated, famfam) { 
+  # we cannot use family()$simulate bc it assumes a fit object as input
+  switch(famfam,
+                    gaussian = rnorm(length(mu),mean=mu,sd=sqrt(phiW)),
+                    poisson = .rpois(length(mu),mu,zero_truncated=zero_truncated), 
+                    binomial = rbinom(length(mu),size=sizes,prob=mu),
+                    Gamma = rgamma(length(mu),shape= mu^2 / phiW, scale=phiW/mu), ## ie shape increase with prior weights, consistent with Gamma()$simulate / spaMM_Gamma()$simulate
+                    COMPoisson = sapply(mu, function(muv) {
+                      lambda <- family$linkfun(muv,log=FALSE)
+                      .COMP_simulate(lambda=lambda,nu=COMP_nu)
+                    }),
+                    negbin = .rnbinom(length(mu),size=NB_shape, mu_str=mu, zero_truncated=zero_truncated), 
+                    stop("(!) random sample from given family not yet implemented")
+  )
+} ## vector-valued function from vector input
 
 # simulate.HLfit(fullm[[2]],newdata=fullm[[1]]$data,size=fullm[[1]]$data$total) for multinomial avec binomial nichées de dimension différentes
 # FR->FR misses the computation of random effects for new spatial positions: cf comments in the code below
@@ -70,42 +73,44 @@ simulate.HLfit <- function(object, nsim = 1, seed = NULL, newdata=NULL,
   if (is.null(sizes)) sizes <- .get_BinomialDen(object)
   nrand <- length(object$ZAlist)
   if (nrand==0L) {
-    mu <- predict(object,newdata=newdata)[,1L]
-    if (nsim>1) mu <- matrix(rep(mu,nsim),ncol=nsim)
+    mu <- predict(object,newdata=newdata,binding=NA)
+    if (nsim>1) mu <- replicate(nsim,mu,simplify=FALSE) # mu <- matrix(rep(mu,nsim),ncol=nsim)
   } else { ## MIXED MODEL
     if (type=="(ranef|response)") { ## Booth & Hobert approximate approach
       if (verbose) cat("Simulation from conditional random effect distribution | observed response:\n") 
-      point_pred_eta <- predict(object,newdata=newdata,variances=list(BH98=TRUE))
+      point_pred_eta <- predict(object,newdata=newdata,variances=list(BH98=TRUE)) ## (with newX.pv=0)
       if (all(attr(object$rand.families,"lcrandfamfam")=="gaussian")){
-        rand_eta <- MASS::mvrnorm(n=nsim,mu=point_pred_eta[,1L],Sigma=attr(point_pred_eta,"predVar"))
+        rand_eta <- mvrnorm(n=nsim,mu=point_pred_eta[,1L],Sigma=attr(point_pred_eta,"predVar"))
         if (nsim>1L) {
           rand_eta <- t(rand_eta)
         } else rand_eta <- rand_eta[1L,]
-        mu <- object$family$linkinv(rand_eta) ## ! freqs for binomial, counts for poisson: suitable for final code
+        if ( ! is.null(zero_truncated <- object$family$zero_truncated)) {
+          mu <- object$family$linkinv(rand_eta,mu_truncated=zero_truncated)
+        } else mu <- object$family$linkinv(rand_eta) ## ! freqs for binomial, counts for poisson: suitable for final code
       } else stop("This conditional simulation is not implemented for non-gaussian random-effects")
     } else if ( type=="residual") { ## conditional on predicted ranefs
       if (verbose) cat("Unconditional simulation given predicted random effects:\n") 
-      mu <- predict(object,newdata=newdata)[,1L]
-      if (nsim>1) mu <- matrix(rep(mu,nsim),ncol=nsim)
+      mu <- predict(object,newdata=newdata,binding=NA)
+      if (nsim>1) mu <- replicate(nsim,mu,simplify=FALSE) #matrix(rep(mu,nsim),ncol=nsim)
     } else if ( type=="marginal"){ ## unconditional MIXED MODEL
       if (verbose) cat("Unconditional simulation:\n") 
-      mu_fixed <- predict(object, newdata=newdata, re.form=NA)
-      eta_fixed <- object$family$linkfun(mu_fixed[,1L])
+      mu_fixed <- predict(object, newdata=newdata, re.form=NA,binding=NA) ## mu_T
+      if ( ! is.null(zero_truncated <- object$family$zero_truncated)) {
+        eta_fixed <- object$family$linkfun(mu_fixed, mu_truncated=zero_truncated) ## back to _U to add ranefs to the linear predictor.
+      } else eta_fixed <- object$family$linkfun(mu_fixed)
       if (is.null(newdata)) {
-        ZAL <- get_ZALMatrix(object,as_matrix=.eval_as_mat_arg(object))
+        ZAL <- get_ZALMatrix(object)
         cum_n_u_h <- attr(object$lambda,"cum_n_u_h")
         vec_n_u_h <- diff(cum_n_u_h)
       } else { ## unconditional MM with newdata
         #   ## [-2] so that HLframes does not try to find the response variables  
         allFrames <- .HLframes(formula=attr(object$predictor,"oriFormula")[-2],data=newdata) 
-        FL <- .spMMFactorList(object$predictor, allFrames$mf, 0L, drop=TRUE) 
-        ##### the following code with NULL LMatrix ignores spatial effects with newdata:
-        ZALlist <- .compute_ZAXlist(XMatrix=NULL,ZAlist=FL$Design)
-        nrand <- length(ZALlist)
-        # to overcome this we needto calculate the unconditional covmat including for the (nex) positions
+        ZALlist <- .spMMFactorList(object$predictor, allFrames$mf, 0L, drop=TRUE,sparse_precision=FALSE) 
+        ##### the following code ignores spatial effects with newdata:
+        # to overcome this we need to calculate the unconditional covmat including for the (new) positions
         vec_n_u_h <- unlist(lapply(ZALlist,ncol)) ## nb cols each design matrix = nb realizations each ranef
         cum_n_u_h <- cumsum(c(0,vec_n_u_h))
-        ZALlist <- lapply(seq_len(length(ZALlist)),as.matrix)
+        ZALlist <- lapply(ZALlist,as.matrix)
         ZAL <- do.call(cbind,ZALlist)
       }
       lcrandfamfam <- attr(object$rand.families,"lcrandfamfam") ## unlist(lapply(object$rand.families, function(rf) {tolower(rf$family)})) 
@@ -127,8 +132,16 @@ simulate.HLfit <- function(object, nsim = 1, seed = NULL, newdata=NULL,
       newV <- do.call(rbind,newV) ## each column a simulation
       if (nsim==1L) {
         eta <- eta_fixed + drop(ZAL %id*% newV) 
-      } else eta <-  matrix(rep(eta_fixed,nsim),ncol=nsim) + as.matrix(ZAL %id*% newV) ## nobs rows, nsim col
-      mu <- object$family$linkinv(eta) 
+        if ( ! is.null(zero_truncated <- object$family$zero_truncated)) {
+          mu <- object$family$linkinv(eta,mu_truncated=zero_truncated)
+        } else mu <- object$family$linkinv(eta) 
+      } else {
+        eta <-  matrix(rep(eta_fixed,nsim),ncol=nsim) + as.matrix(ZAL %id*% newV) ## nobs rows, nsim col
+        mu <- vector("list",nsim)
+        if ( ! is.null(zero_truncated <- object$family$zero_truncated)) {
+          for (it in seq_len(nsim)) mu[[it]] <- object$family$linkinv(eta[,it],mu_truncated=zero_truncated)
+        } else for (it in seq_len(nsim)) mu[[it]] <- object$family$linkinv(eta[,it]) 
+      }
     } else stop("Unknown simulate 'type' value.")
   }
   ## ! mu := freqs for binomial, counts for poisson ## vector or matrix
@@ -138,13 +151,18 @@ simulate.HLfit <- function(object, nsim = 1, seed = NULL, newdata=NULL,
   # resu <- object$family$simulate(object,nsim=nsim) ## could be OK and more efficient for CONDITIONAL simulation
   if (famfam =="COMPoisson") {
     COMP_nu <- environment(object$family$aic)$nu
-  } else if (famfam =="negbin") {
+  } else if (famfam == "negbin") {
     NB_shape <- environment(object$family$aic)$shape
-  } 
-  if (nsim>1) { ## then mu has several columns
-    resu <- apply(mu,2, .r_resid_var, phiW=phiW,sizes=sizes,COMP_nu=COMP_nu,NB_shape=NB_shape,famfam=famfam) ## matrix
+    zero_truncated <- identical(object$family$zero_truncated,TRUE)
+  } else if (famfam == "poisson") {
+    zero_truncated <- identical(object$family$zero_truncated,TRUE)
+  }
+  if (nsim>1) { ## matrix output, but list input for mu, as it is easier to keep attributes on mu by using a list.
+    resu <- sapply(mu, .r_resid_var, phiW=phiW,sizes=sizes,COMP_nu=COMP_nu,NB_shape=NB_shape, zero_truncated=zero_truncated,
+                                         famfam=famfam) 
   } else {
-    resu <- .r_resid_var(mu,phiW=phiW,sizes=sizes,COMP_nu=COMP_nu,NB_shape=NB_shape,famfam=famfam)
+    resu <- .r_resid_var(mu,phiW=phiW,sizes=sizes,COMP_nu=COMP_nu,NB_shape=NB_shape, zero_truncated=zero_truncated,
+                         famfam=famfam)
   }
   return(resu)    
 }
@@ -166,10 +184,7 @@ simulate.HLfitlist <- function(object,nsim=1,seed=NULL,newdata=object[[1]]$data,
     allrownames <- unique(unlist(lapply(object, function(hl){rownames(hl$data)})))
     resu <- matrix(0,nrow=length(allrownames),ncol=length(object)) ## two cols if 3 types
     cumul <- 0
-    if (length(sizes) != nrow(newdata)) {
-      mess <- pastefrom("length(sizes) != nrow(newdata).",prefix="(!) From ")
-      stop(mess)
-    }
+    if (length(sizes) != nrow(newdata)) stop("length(sizes) != nrow(newdata).")
     for (it in seq(ncol(resu))) {
       ## it = 1 se ramène à simulate(object[[1]])
       if (is.null(sizes)) sizes <- .get_BinomialDen(object[[it]])
